from flask import Blueprint, jsonify, request, render_template
from controllers.message_controller import MessageController
from models.message import Message
from models.base import Session
from app.middleware.auth import auth_required
import logging
from threading import Lock

logger = logging.getLogger(__name__)
message_bp = Blueprint('messages', __name__)
fetch_lock = Lock()

@message_bp.route("/messages-by-author")
def messages_by_author():
    page = int(request.args.get('page', 1))
    result = MessageController.get_messages_by_author(page)
    return jsonify(result) if result else (jsonify({'error': 'Error loading messages'}), 500)

@message_bp.route("/messages")
def messages_page():
    session = Session()
    try:
        total_messages = session.query(Message).count()
        return render_template('messages.html', total_messages=total_messages)
    finally:
        session.close()

@message_bp.route("/fetch-messages", methods=['POST'])
def fetch_messages():
    if not fetch_lock.acquire(blocking=False):
        return jsonify({
            "status": "error",
            "message": "A fetch operation is already in progress",
            "code": "FETCH_IN_PROGRESS"
        }), 409

    try:
        logger.info("Starting manual fetch operation")
        chats_count = MessageController.process_chats()
        new_messages_count = MessageController.process_new_messages()
        duplicate_count = MessageController.get_duplicate_count()
        
        return jsonify({
            "status": "success",
            "data": {
                "new_messages": new_messages_count,
                "new_chats": chats_count,
                "duplicate_messages": duplicate_count
            },
            "message": "Fetch operation completed successfully"
        })
    except Exception as e:
        logger.error(f"Error in fetch_messages: {str(e)}", exc_info=True)
        return jsonify({
            "status": "error",
            "message": str(e),
            "code": "FETCH_ERROR"
        }), 500
    finally:
        fetch_lock.release()

@message_bp.route("/messages-json")
def messages_json():
    try:
        page = int(request.args.get('page', 1))
        result = MessageController.get_messages(page)
        return jsonify(result) if result else (jsonify({'error': 'Failed to load messages'}), 500)
    except Exception as e:
        logger.exception("Error in messages_json endpoint")
        return jsonify({'error': str(e)}), 500